"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _moment = _interopRequireDefault(require("moment"));

var _ajv = _interopRequireDefault(require("ajv"));

var _file = _interopRequireDefault(require("formidable/lib/file"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const converts = {
  'date': v => _moment.default.utc(v, 'YYYY-MM-DD').toDate(),
  'time': v => _moment.default.utc(v, 'HH:mm:ssZ.SSS').toDate(),
  'date-time': v => _moment.default.utc(v).toDate()
};

class Validator {
  constructor() {
    this.ajv = new _ajv.default({
      coerceTypes: true,
      useDefaults: true
    });
    this.ajv.addKeyword('convert', {
      compile(convert, schema) {
        const fn = convert === true ? converts[schema.format] : convert;

        if (fn && typeof fn === 'function') {
          return (value, fullKey, data, key) => {
            data[key] = fn(value);
            return true;
          };
        }

        return () => true;
      }

    });
    this.ajv.addKeyword('file', {
      compile(checkFile, schema) {
        if (checkFile) {
          return value => {
            if (value && value instanceof _file.default) {
              return true;
            }

            return false;
          };
        }

        return () => true;
      }

    });
    this.schemas = {};
  }

  getSchemas() {
    return this.schemas;
  }

  addSchema(schemaName, props, options = {}) {
    (0, _utils.assert)(schemaName, 'schemaName is required');

    const _transformExtends = (0, _utils.transformExtends)(schemaName),
          name = _transformExtends.name,
          parents = _transformExtends.parents;

    let schema = (0, _utils.propsToSchema)(props, options);

    if (parents.length) {
      if (schema) {
        parents.push(schema);
      }

      if (parents.length === 1) {
        schema = _objectSpread({}, options, parents[0]);
      } else {
        schema = _objectSpread({}, options, {
          allOf: parents
        });
      }
    } else {
      schema = _objectSpread({}, options, schema);
    }

    this.schemas[name] = schema;
    this.ajv.addSchema(schema, (0, _utils.ref)(name));
  }

  compile(schema) {
    return this.ajv.compile(schema);
  }

}

exports.default = Validator;